# Copyright (C) 2020 FUJITSU
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from kubernetes import client
import os
from unittest import mock

from tacker.common import exceptions
from tacker.tests.unit import base
from tacker.tests.unit import fake_request
from tacker.tests.unit.vnfm.infra_drivers.kubernetes import fakes
from tacker.vnfm.infra_drivers.kubernetes.k8s import tosca_kube_object
from tacker.vnfm.infra_drivers.kubernetes.k8s import translate_outputs


class TestTransformer(base.TestCase):
    def setUp(self):
        super(TestTransformer, self).setUp()
        self.yaml_path = os.path.join(
            os.path.dirname(os.path.abspath(__file__)),
            "kubernetes_api_resource/")
        self.k8s_client_dict = fakes.fake_k8s_client_dict()
        self.transfromer = translate_outputs.Transformer(
            client.CoreV1Api,
            client.AppsV1Api,
            client.AutoscalingApi,
            self.k8s_client_dict)

    def test_deploy_k8s_create_false(self):
        kubernetes_objects = []
        k8s_obj = fakes.fake_k8s_dict()
        kubernetes_objects.append(k8s_obj)
        self.assertRaises(exceptions.CreateApiFalse,
                          self.transfromer.deploy_k8s,
                          kubernetes_objects)

    @mock.patch.object(translate_outputs.Transformer,
                       "_select_k8s_client_and_api")
    def test_deploy_k8s(self, mock_k8s_client_and_api):
        req = fake_request.HTTPRequest.blank(
            'apis/apps/v1/namespaces/curryns/deployments')
        mock_k8s_client_and_api.return_value = req
        kubernetes_objects = []
        k8s_obj = fakes.fake_k8s_dict()
        kubernetes_objects.append(k8s_obj)
        new_k8s_objs = self.transfromer.deploy_k8s(kubernetes_objects)
        self.assertIsInstance(new_k8s_objs, list)
        self.assertIsNotNone(new_k8s_objs)
        self.assertEqual(new_k8s_objs[0]['status'], 'Creating')

    def test_deployment(self):
        k8s_objs = self.transfromer.get_k8s_objs_from_yaml(
            ['deployment.yaml'], self.yaml_path, '')
        k8s_obj = k8s_objs[0].get('object')
        self.assertIsNotNone(k8s_obj)
        self.assertEqual(k8s_objs[0].get('namespace'), '')
        self.assertEqual(k8s_obj.kind, 'Deployment')
        self.assertEqual(k8s_obj.api_version, 'apps/v1')

        # V1DeploymentCondition
        self.assertEqual(k8s_obj.status.conditions[0].status, True)
        self.assertEqual(k8s_obj.status.conditions[0].type, 'Deployment')
        # V1DeploymentSpec
        self.assertIsNotNone(k8s_obj.spec.selector)
        self.assertIsNotNone(k8s_obj.spec.template)
        # V1LabelSelectorRequirement
        self.assertEqual(k8s_obj.spec.selector.
                         match_expressions[0].key, 'test')
        self.assertEqual(k8s_obj.spec.selector.
                         match_expressions[0].operator, 'test')

    def test_api_service(self):
        k8s_objs = self.transfromer.get_k8s_objs_from_yaml(
            ['api-service.yaml'], self.yaml_path, ''
        )
        k8s_obj = k8s_objs[0].get('object')
        self.assertIsNotNone(k8s_obj)
        self.assertEqual(k8s_objs[0].get('namespace'), '')
        self.assertEqual(k8s_obj.kind, 'APIService')
        self.assertEqual(k8s_obj.api_version, 'apiregistration.k8s.io/v1')
        # V1APIServiceSpec
        self.assertEqual(k8s_obj.spec.group_priority_minimum, 17000)
        self.assertIsNotNone(k8s_obj.spec.service)
        self.assertEqual(k8s_obj.spec.version_priority, 5)

    def test_cluster_role(self):
        k8s_objs = self.transfromer.get_k8s_objs_from_yaml(
            ['cluster-role.yaml'], self.yaml_path, ''
        )
        k8s_obj = k8s_objs[0].get('object')
        self.assertIsNotNone(k8s_obj)
        self.assertEqual(k8s_objs[0].get('namespace'), '')
        self.assertEqual(k8s_obj.kind, 'ClusterRole')
        self.assertEqual(k8s_obj.api_version, 'rbac.authorization.k8s.io/v1')
        # V1PolicyRule
        self.assertIsNotNone(k8s_obj.rules[0].verbs)

    def test_cluster_role_binding(self):
        k8s_objs = self.transfromer.get_k8s_objs_from_yaml(
            ['cluster-role-binding.yaml'], self.yaml_path, ''
        )
        k8s_obj = k8s_objs[0].get('object')
        self.assertIsNotNone(k8s_obj)
        self.assertEqual(k8s_objs[0].get('namespace'), '')
        self.assertEqual(k8s_obj.kind, 'ClusterRoleBinding')
        self.assertEqual(k8s_obj.api_version, 'rbac.authorization.k8s.io/v1')
        # V1ClusterRoleBinding
        self.assertIsNotNone(k8s_obj.role_ref)
        # V1RoleRef
        self.assertEqual(k8s_obj.role_ref.api_group,
                         'rbac.authorization.k8s.io')
        self.assertEqual(k8s_obj.role_ref.kind, 'ClusterRole')
        self.assertEqual(k8s_obj.role_ref.name, 'curry-cluster-role')
        # V1Subject
        self.assertEqual(k8s_obj.subjects[0].kind, 'ServiceAccount')
        self.assertEqual(k8s_obj.subjects[0].name, 'curry-cluster-sa')

    def test_config_map(self):
        k8s_objs = self.transfromer.get_k8s_objs_from_yaml(
            ['config-map.yaml'], self.yaml_path, 'curryns'
        )
        k8s_obj = k8s_objs[0].get('object')
        self.assertIsNotNone(k8s_obj)
        self.assertEqual(k8s_objs[0].get('namespace'), 'curryns')
        self.assertEqual(k8s_obj.kind, 'ConfigMap')
        self.assertEqual(k8s_obj.api_version, 'v1')

    def test_daemon_set(self):
        k8s_objs = self.transfromer.get_k8s_objs_from_yaml(
            ['daemon-set.yaml'], self.yaml_path, ''
        )
        k8s_obj = k8s_objs[0].get('object')
        self.assertIsNotNone(k8s_obj)
        self.assertEqual(k8s_objs[0].get('namespace'), '')
        self.assertEqual(k8s_obj.kind, 'DaemonSet')
        self.assertEqual(k8s_obj.api_version, 'apps/v1')
        # V1DaemonSetStatus
        self.assertEqual(k8s_obj.status.current_number_scheduled, 1)
        self.assertEqual(k8s_obj.status.desired_number_scheduled, 1)
        self.assertEqual(k8s_obj.status.number_misscheduled, 1)
        self.assertEqual(k8s_obj.status.number_ready, 1)
        # V1DaemonSetCondition
        self.assertEqual(k8s_obj.status.conditions[0].status, True)
        self.assertEqual(k8s_obj.status.conditions[0].type, 'DaemonSet')
        # V1DaemonSetSpec
        self.assertIsNotNone(k8s_obj.spec.selector)
        self.assertIsNotNone(k8s_obj.spec.template)

    def test_horizontal_pod_autoscaler(self):
        k8s_objs = self.transfromer.get_k8s_objs_from_yaml(
            ['horizontal-pod-autoscaler.yaml'], self.yaml_path, 'default'
        )
        k8s_obj = k8s_objs[0].get('object')
        self.assertIsNotNone(k8s_obj)
        self.assertEqual(k8s_objs[0].get('namespace'), 'default')
        self.assertEqual(k8s_obj.kind, 'HorizontalPodAutoscaler')
        self.assertEqual(k8s_obj.api_version, 'autoscaling/v1')
        # V1HorizontalPodAutoscalerSpec
        self.assertEqual(k8s_obj.spec.max_replicas, 3)
        self.assertIsNotNone(k8s_obj.spec.scale_target_ref)
        # V1CrossVersionObjectReference
        self.assertEqual(k8s_obj.spec.scale_target_ref.kind, 'Deployment')
        self.assertEqual(k8s_obj.spec.scale_target_ref.name,
                         'curry-svc-vdu001')
        # V1HorizontalPodAutoscalerStatus
        self.assertEqual(k8s_obj.status.current_replicas, 1)
        self.assertEqual(k8s_obj.status.desired_replicas, 1)

    def test_job(self):
        k8s_objs = self.transfromer.get_k8s_objs_from_yaml(
            ['job.yaml'], self.yaml_path, ''
        )
        k8s_obj = k8s_objs[0].get('object')
        self.assertIsNotNone(k8s_obj)
        self.assertEqual(k8s_objs[0].get('namespace'), '')
        self.assertEqual(k8s_obj.kind, 'Job')
        self.assertEqual(k8s_obj.api_version, 'batch/v1')
        # V1JobCondition
        self.assertEqual(k8s_obj.status.conditions[0].status, True)
        self.assertEqual(k8s_obj.status.conditions[0].type, 'Job')
        # V1JobSpec
        self.assertIsNotNone(k8s_obj.spec.template)

    def test_lease(self):
        k8s_objs = self.transfromer.get_k8s_objs_from_yaml(
            ['lease.yaml'], self.yaml_path, 'default'
        )
        k8s_obj = k8s_objs[0].get('object')
        self.assertIsNotNone(k8s_obj)
        self.assertEqual(k8s_objs[0].get('namespace'), 'default')
        self.assertEqual(k8s_obj.kind, 'Lease')
        self.assertEqual(k8s_obj.api_version, 'coordination.k8s.io/v1')

    def test_local_subject_access_review(self):
        k8s_objs = self.transfromer.get_k8s_objs_from_yaml(
            ['local-subject-access-review.yaml'], self.yaml_path, 'curry-ns'
        )
        k8s_obj = k8s_objs[0].get('object')
        self.assertIsNotNone(k8s_obj)
        self.assertEqual(k8s_objs[0].get('namespace'), 'curry-ns')
        self.assertEqual(k8s_obj.kind, 'LocalSubjectAccessReview')
        self.assertEqual(k8s_obj.api_version, 'authorization.k8s.io/v1')
        self.assertIsNotNone(k8s_obj.spec)

    def test_namespace(self):
        k8s_objs = self.transfromer.get_k8s_objs_from_yaml(
            ['namespace.yaml'], self.yaml_path, ''
        )
        k8s_obj = k8s_objs[0].get('object')
        self.assertIsNotNone(k8s_obj)
        self.assertEqual(k8s_objs[0].get('namespace'), '')
        self.assertEqual(k8s_obj.kind, 'Namespace')
        self.assertEqual(k8s_obj.api_version, 'v1')

        # V1NamespaceCondition
        self.assertEqual(k8s_obj.status.conditions[0].status, True)
        self.assertEqual(k8s_obj.status.conditions[0].type, 'Namespace')

    def test_network_policy(self):
        k8s_objs = self.transfromer.get_k8s_objs_from_yaml(
            ['network-policy.yaml'], self.yaml_path, ''
        )
        k8s_obj = k8s_objs[0].get('object')
        self.assertIsNotNone(k8s_obj)
        self.assertEqual(k8s_objs[0].get('namespace'), '')
        self.assertEqual(k8s_obj.kind, 'NetworkPolicy')
        self.assertEqual(k8s_obj.api_version, 'networking.k8s.io/v1')

        # V1IPBlock
        self.assertEqual(k8s_obj.spec.egress[0].to[0].ip_block.cidr,
                         '10.0.0.0/24')
        # V1NetworkPolicySpec
        self.assertIsNotNone(k8s_obj.spec.pod_selector)

    def test_node(self):
        k8s_objs = self.transfromer.get_k8s_objs_from_yaml(
            ['node.yaml'], self.yaml_path, ''
        )
        k8s_obj = k8s_objs[0].get('object')
        self.assertIsNotNone(k8s_obj)
        self.assertEqual(k8s_objs[0].get('namespace'), '')
        self.assertEqual(k8s_obj.kind, 'Node')
        self.assertEqual(k8s_obj.api_version, 'v1')

        # V1ConfigMapNodeConfigSource
        self.assertEqual(k8s_obj.spec.config_source.
                         config_map.kubelet_config_key, 'kubelet')
        self.assertEqual(k8s_obj.spec.config_source.
                         config_map.namespace, 'kube-system')
        self.assertEqual(k8s_obj.spec.config_source.
                         config_map.name, 'CONFIG_MAP_NAME')
        # V1Taint
        self.assertEqual(k8s_obj.spec.taints[0].key, 'test')
        self.assertEqual(k8s_obj.spec.taints[0].effect, 'test')
        # V1NodeAddress
        self.assertEqual(k8s_obj.status.addresses[0].address, '1.1.1.1')
        self.assertEqual(k8s_obj.status.addresses[0].type, 'test')
        # V1NodeCondition
        self.assertEqual(k8s_obj.status.conditions[0].status, True)
        self.assertEqual(k8s_obj.status.conditions[0].type, 'Node')
        # V1DaemonEndpoint
        self.assertEqual(k8s_obj.status.daemon_endpoints.
                         kubelet_endpoint.port, 8080)
        # V1ContainerImage
        self.assertEqual(k8s_obj.status.images[0].names, 'test')
        # V1NodeSystemInfo
        self.assertEqual(k8s_obj.status.node_info.architecture, 'test')
        self.assertEqual(k8s_obj.status.node_info.boot_id, 'test')
        self.assertEqual(k8s_obj.status.node_info.
                         container_runtime_version, 'test')
        self.assertEqual(k8s_obj.status.node_info.kube_proxy_version, 'test')
        self.assertEqual(k8s_obj.status.node_info.kubelet_version, 'test')
        self.assertEqual(k8s_obj.status.node_info.machine_id, 'test')
        self.assertEqual(k8s_obj.status.node_info.operating_system, 'test')
        self.assertEqual(k8s_obj.status.node_info.os_image, 'test')
        self.assertEqual(k8s_obj.status.node_info.system_uuid, 'test')
        # V1AttachedVolume
        self.assertEqual(k8s_obj.status.volumes_attached[0].
                         device_path, 'test')
        self.assertEqual(k8s_obj.status.volumes_attached[0].name, 'test')

    def test_persistent_volume(self):
        k8s_objs = self.transfromer.get_k8s_objs_from_yaml(
            ['persistent-volume.yaml'], self.yaml_path, ''
        )
        k8s_obj = k8s_objs[0].get('object')
        self.assertIsNotNone(k8s_obj)
        self.assertEqual(k8s_objs[0].get('namespace'), '')
        self.assertEqual(k8s_obj.kind, 'PersistentVolume')
        self.assertEqual(k8s_obj.api_version, 'v1')
        # V1AzureFilePersistentVolumeSource
        self.assertEqual(k8s_obj.spec.azure_file.secret_name, 'azure-secret')
        self.assertEqual(k8s_obj.spec.azure_file.share_name, 'aksshare')
        # V1CephFSPersistentVolumeSource
        self.assertEqual(k8s_obj.spec.cephfs.monitors[0], '10.16.154.78:6789')
        # V1CinderPersistentVolumeSource
        self.assertEqual(k8s_obj.spec.cinder.volume_id,
                         '90d6900d-808f-4ddb-a30e-5ef821f58b4e')
        # V1CSIPersistentVolumeSource
        self.assertEqual(k8s_obj.spec.csi.driver, 'csi-nfsplugin')
        self.assertEqual(k8s_objs[0].get('object').spec.csi.volume_handle,
                         'data-id')
        # V1FlexPersistentVolumeSource
        self.assertEqual(k8s_objs[0].get('object').spec.flex_volume.driver,
                         'kubernetes.io/lvm')
        # V1GlusterfsPersistentVolumeSource
        self.assertEqual(k8s_objs[0].get('object').spec.glusterfs.endpoints,
                         'glusterfs-cluster')
        self.assertEqual(k8s_obj.spec.glusterfs.path, 'kube_vol')
        # V1ISCSIPersistentVolumeSource
        self.assertEqual(k8s_obj.spec.iscsi.target_portal, '10.0.2.15:3260')
        self.assertEqual(k8s_obj.spec.iscsi.iqn,
                         'iqn.2001-04.com.example:storage.kube.sys1.xyz')
        self.assertEqual(k8s_obj.spec.iscsi.lun, 0)
        # V1LocalVolumeSource
        self.assertEqual(k8s_obj.spec.local.path, '/mnt/disks/ssd1')
        # V1RBDPersistentVolumeSource
        self.assertEqual(k8s_obj.spec.rbd.monitors[0], '10.16.154.78:6789')
        self.assertEqual(k8s_obj.spec.rbd.image, 'foo')
        # V1ScaleIOPersistentVolumeSource
        self.assertEqual(k8s_obj.spec.scale_io.gateway,
                         'https://localhost:443/api')
        self.assertIsNotNone(k8s_obj.spec.scale_io.secret_ref)
        self.assertEqual(k8s_obj.spec.scale_io.system, 'scaleio')
        # V1AWSElasticBlockStoreVolumeSource
        self.assertEqual(k8s_obj.spec.aws_elastic_block_store.volume_id,
                         '123')
        # V1AzureDiskVolumeSource
        self.assertEqual(k8s_obj.spec.azure_disk.disk_name, 'test.vhd')
        self.assertEqual(
            k8s_obj.spec.azure_disk.disk_uri,
            'https://someaccount.blob.microsoft.net/vhds/test.vhd')

    def test_persistent_volume_claim(self):
        k8s_objs = self.transfromer.get_k8s_objs_from_yaml(
            ['persistent-volume-claim.yaml'], self.yaml_path, ''
        )
        k8s_obj = k8s_objs[0].get('object')
        self.assertIsNotNone(k8s_obj)
        self.assertEqual(k8s_objs[0].get('namespace'), '')
        self.assertEqual(k8s_obj.kind, 'PersistentVolumeClaim')
        self.assertEqual(k8s_obj.api_version, 'v1')
        # V1PersistentVolumeClaimCondition
        self.assertEqual(k8s_obj.status.conditions[0].status, True)
        self.assertEqual(k8s_obj.status.conditions[0].type,
                         'PersistentVolumeClaim')
        # V1TypedLocalObjectReference
        self.assertEqual(k8s_obj.spec.data_source.name,
                         'existing-src-pvc-name')
        self.assertEqual(k8s_obj.spec.data_source.kind,
                         'PersistentVolumeClaim')

    def test_pod(self):
        k8s_objs = self.transfromer.get_k8s_objs_from_yaml(
            ['pod.yaml'], self.yaml_path, ''
        )
        k8s_obj = k8s_objs[0].get('object')
        self.assertIsNotNone(k8s_obj)
        self.assertEqual(k8s_objs[0].get('namespace'), '')
        self.assertEqual(k8s_obj.kind, 'Pod')
        self.assertEqual(k8s_obj.api_version, 'v1')
        # V1NodeSelector
        self.assertIsNotNone(
            k8s_obj.spec.affinity.node_affinity.
            required_during_scheduling_ignored_during_execution.
            node_selector_terms)
        # V1NodeSelectorRequirement
        self.assertEqual(
            k8s_obj.spec.affinity.node_affinity.
            required_during_scheduling_ignored_during_execution.
            node_selector_terms[0].match_expressions[0].key,
            'kubernetes.io/e2e-az-name')
        self.assertEqual(
            k8s_obj.spec.affinity.node_affinity.
            required_during_scheduling_ignored_during_execution.
            node_selector_terms[0].match_expressions[0].operator,
            'In')
        # V1PreferredSchedulingTerm
        self.assertEqual(
            k8s_obj.spec.affinity.node_affinity.
            preferred_during_scheduling_ignored_during_execution[0].
            weight, 1)
        self.assertIsNotNone(
            k8s_obj.spec.affinity.node_affinity.
            preferred_during_scheduling_ignored_during_execution[0].
            preference)
        # V1PodAffinityTerm
        self.assertEqual(
            k8s_obj.spec.affinity.pod_anti_affinity.
            preferred_during_scheduling_ignored_during_execution[0].
            pod_affinity_term.topology_key, 'topology.kubernetes.io/zone')
        # V1WeightedPodAffinityTerm
        self.assertEqual(
            k8s_obj.spec.affinity.pod_anti_affinity.
            preferred_during_scheduling_ignored_during_execution[0].
            weight, 100)
        self.assertIsNotNone(
            k8s_obj.spec.affinity.pod_anti_affinity.
            preferred_during_scheduling_ignored_during_execution[0].
            pod_affinity_term)
        # V1OwnerReference
        self.assertEqual(k8s_obj.metadata.owner_references[0].api_version,
                         'apps/v1')
        self.assertEqual(k8s_obj.metadata.owner_references[0].kind,
                         'ReplicaSet')
        self.assertEqual(k8s_obj.metadata.owner_references[0].name,
                         'my-repset')
        self.assertEqual(k8s_obj.metadata.owner_references[0].uid,
                         'd9607e19-f88f-11e6-a518-42010a800195')
        # V1HTTPHeader
        self.assertEqual(k8s_obj.spec.containers[0].liveness_probe.http_get.
                         http_headers[0].name, 'Custom-Header')
        self.assertEqual(k8s_obj.spec.containers[0].liveness_probe.http_get.
                         http_headers[0].value, 'Awesome')
        # V1TCPSocketAction
        self.assertEqual(k8s_obj.spec.containers[0].liveness_probe.
                         tcp_socket.port, 8080)
        # V1VolumeDevice
        self.assertEqual(k8s_obj.spec.containers[0].volume_devices[0].
                         device_path, '/dev/xvda')
        self.assertEqual(k8s_obj.spec.containers[0].volume_devices[0].name,
                         'data')
        # V1PodReadinessGate
        self.assertEqual(k8s_obj.spec.readiness_gates[0].condition_type,
                         'www.example.com/feature-1')
        # V1Sysctl
        self.assertEqual(k8s_obj.spec.security_context.sysctls[0].name,
                         'kernel.shm_rmid_forced')
        self.assertEqual(k8s_obj.spec.security_context.sysctls[0].value, '0')
        # V1ContainerStateTerminated
        self.assertEqual(k8s_obj.status.container_statuses[0].last_state.
                         terminated.exit_code, 1)
        # V1EphemeralContainer
        self.assertEqual(k8s_obj.spec.topology_spread_constraints[0].
                         topology_key, 'zone')
        # V1TopologySpreadConstraint
        self.assertEqual(k8s_obj.spec.ephemeral_containers[0].name,
                         'debugger')
        # V1HTTPGetAction
        self.assertEqual(k8s_obj.spec.containers[0].liveness_probe.
                         http_get.port, 8080)
        # V1ConfigMapKeySelector
        self.assertEqual(k8s_obj.spec.containers[0].env[0].value_from.
                         config_map_key_ref.key, 'test')
        # V1EnvVar
        self.assertEqual(k8s_obj.spec.containers[0].env[0].name, 'test')
        # V1SecretKeySelector
        self.assertEqual(k8s_obj.spec.containers[0].env[0].value_from.
                         secret_key_ref.key, 'test')
        # V1ContainerPort
        self.assertEqual(k8s_obj.spec.containers[0].ports[0].
                         container_port, 8080)
        # V1VolumeMount
        self.assertEqual(k8s_obj.spec.containers[0].volume_mounts[0].
                         mount_path, '/data/redis')
        self.assertEqual(k8s_obj.spec.containers[0].volume_mounts[0].
                         name, 'redis-storage')
        # V1PodCondition
        self.assertEqual(k8s_obj.status.conditions[0].status, True)
        self.assertEqual(k8s_obj.status.conditions[0].type, 'Pod')
        # V1ContainerStatus
        self.assertEqual(k8s_obj.status.container_statuses[0].image, 'test')
        self.assertEqual(k8s_obj.status.container_statuses[0].image_id, 123)
        self.assertEqual(k8s_obj.status.container_statuses[0].name, 'test')
        self.assertEqual(k8s_obj.status.container_statuses[0].ready, True)
        self.assertEqual(k8s_obj.status.container_statuses[0].restart_count, 1)

    def test_priority_class(self):
        k8s_objs = self.transfromer.get_k8s_objs_from_yaml(
            ['priority-class.yaml'], self.yaml_path, ''
        )
        k8s_obj = k8s_objs[0].get('object')
        self.assertIsNotNone(k8s_obj)
        self.assertEqual(k8s_objs[0].get('namespace'), '')
        self.assertEqual(k8s_obj.kind, 'PriorityClass')
        self.assertEqual(k8s_obj.api_version, 'scheduling.k8s.io/v1')
        # V1PriorityClass
        self.assertEqual(k8s_obj.value, 1000000)

    def test_replica_set(self):
        k8s_objs = self.transfromer.get_k8s_objs_from_yaml(
            ['replica-set.yaml'], self.yaml_path, ''
        )
        k8s_obj = k8s_objs[0].get('object')
        self.assertIsNotNone(k8s_obj)
        self.assertEqual(k8s_objs[0].get('namespace'), '')
        self.assertEqual(k8s_obj.kind, 'ReplicaSet')
        self.assertEqual(k8s_obj.api_version, 'apps/v1')

        # V1ReplicaSetStatus
        self.assertEqual(k8s_obj.status.replicas, 1)
        # V1ReplicaSetCondition
        self.assertEqual(k8s_obj.status.conditions[0].status, True)
        self.assertEqual(k8s_obj.status.conditions[0].type, 'ReplicaSet')
        # V1ReplicaSetSpec
        self.assertIsNotNone(k8s_obj.spec.selector)
        self.assertIsNotNone(k8s_obj.spec.template)

    def test_resource_quota(self):
        k8s_objs = self.transfromer.get_k8s_objs_from_yaml(
            ['resource-quota.yaml'], self.yaml_path, 'curryns'
        )
        k8s_obj = k8s_objs[0].get('object')
        self.assertIsNotNone(k8s_obj)
        self.assertEqual(k8s_objs[0].get('namespace'), 'curryns')
        self.assertEqual(k8s_obj.kind, 'ResourceQuota')
        self.assertEqual(k8s_obj.api_version, 'v1')
        # V1ScopedResourceSelectorRequirement
        self.assertEqual(k8s_obj.spec.scope_selector.
                         match_expressions[0].operator, 'In')
        self.assertEqual(k8s_obj.spec.scope_selector.
                         match_expressions[0].scope_name, 'PriorityClass')

    def test_role(self):
        k8s_objs = self.transfromer.get_k8s_objs_from_yaml(
            ['role.yaml'], self.yaml_path, 'curry-ns'
        )
        k8s_obj = k8s_objs[0].get('object')
        self.assertIsNotNone(k8s_obj)
        self.assertEqual(k8s_objs[0].get('namespace'), 'curry-ns')
        self.assertEqual(k8s_obj.kind, 'Role')
        self.assertEqual(k8s_obj.api_version, 'rbac.authorization.k8s.io/v1')

    def test_role_binding(self):
        k8s_objs = self.transfromer.get_k8s_objs_from_yaml(
            ['role-bindings.yaml'], self.yaml_path, 'curry-ns'
        )
        k8s_obj = k8s_objs[0].get('object')
        self.assertIsNotNone(k8s_obj)
        self.assertEqual(k8s_objs[0].get('namespace'), 'curry-ns')
        self.assertEqual(k8s_obj.kind, 'RoleBinding')
        self.assertEqual(k8s_obj.api_version, 'rbac.authorization.k8s.io/v1')
        # V1RoleBinding
        self.assertIsNotNone(k8s_obj.role_ref)

    def test_secret(self):
        k8s_objs = self.transfromer.get_k8s_objs_from_yaml(
            ['secret.yaml'], self.yaml_path, 'default'
        )
        k8s_obj = k8s_objs[0].get('object')
        self.assertIsNotNone(k8s_obj)
        self.assertEqual(k8s_objs[0].get('namespace'), 'default')
        self.assertEqual(k8s_obj.kind, 'Secret')
        self.assertEqual(k8s_obj.api_version, 'v1')

    def test_self_subject_access_review(self):
        k8s_objs = self.transfromer.get_k8s_objs_from_yaml(
            ['self-subject-access-review.yaml'], self.yaml_path, ''
        )
        k8s_obj = k8s_objs[0].get('object')
        self.assertIsNotNone(k8s_obj)
        self.assertEqual(k8s_objs[0].get('namespace'), '')
        self.assertEqual(k8s_obj.kind, 'SelfSubjectAccessReview')
        self.assertEqual(k8s_obj.api_version, 'authorization.k8s.io/v1')
        # V1SelfSubjectAccessReview
        self.assertIsNotNone(k8s_obj.spec)

    def test_self_subject_rules_review(self):
        k8s_objs = self.transfromer.get_k8s_objs_from_yaml(
            ['self-subject-rule-review.yaml'], self.yaml_path, ''
        )
        k8s_obj = k8s_objs[0].get('object')
        self.assertIsNotNone(k8s_obj)
        self.assertEqual(k8s_objs[0].get('namespace'), '')
        self.assertEqual(k8s_obj.kind, 'SelfSubjectRulesReview')
        self.assertEqual(k8s_obj.api_version, 'authorization.k8s.io/v1')
        # V1ResourceRule
        self.assertEqual(k8s_obj.status.resource_rules[0].verbs[0], 'test')
        # V1SelfSubjectRulesReview
        self.assertIsNotNone(k8s_obj.spec)
        # V1SubjectRulesReviewStatus
        self.assertIsNotNone(k8s_obj.status.resource_rules)
        self.assertIsNotNone(k8s_obj.status.non_resource_rules)
        self.assertEqual(k8s_obj.status.incomplete, True)
        # V1NonResourceRule
        self.assertEqual(k8s_obj.status.non_resource_rules[0].verbs[0], 'test')

    def test_service(self):
        k8s_objs = self.transfromer.get_k8s_objs_from_yaml(
            ['service.yaml'], self.yaml_path, 'default'
        )
        k8s_obj = k8s_objs[0].get('object')
        self.assertIsNotNone(k8s_obj)
        self.assertEqual(k8s_objs[0].get('namespace'), 'default')
        self.assertEqual(k8s_obj.kind, 'Service')
        self.assertEqual(k8s_obj.api_version, 'v1')
        # V1ServicePort
        self.assertEqual(k8s_obj.spec.ports[0].port, 80)

    def test_service_account(self):
        k8s_objs = self.transfromer.get_k8s_objs_from_yaml(
            ['service-account.yaml'], self.yaml_path, 'default'
        )
        k8s_obj = k8s_objs[0].get('object')
        self.assertIsNotNone(k8s_obj)
        self.assertEqual(k8s_objs[0].get('namespace'), 'default')
        self.assertEqual(k8s_obj.kind, 'ServiceAccount')
        self.assertEqual(k8s_obj.api_version, 'v1')

    def test_stateful_set(self):
        k8s_objs = self.transfromer.get_k8s_objs_from_yaml(
            ['stateful-set.yaml'], self.yaml_path, ''
        )
        k8s_obj = k8s_objs[0].get('object')
        self.assertIsNotNone(k8s_obj)
        self.assertEqual(k8s_objs[0].get('namespace'), '')
        self.assertEqual(k8s_obj.kind, 'StatefulSet')
        self.assertEqual(k8s_obj.api_version, 'apps/v1')
        # V1StatefulSetSpec
        self.assertIsNotNone(k8s_obj.spec.selector)
        self.assertIsNotNone(k8s_obj.spec.template)
        self.assertEqual(k8s_obj.spec.service_name, 'nginx')
        # V1StatefulSetCondition
        self.assertEqual(k8s_obj.status.conditions[0].status, True)
        self.assertEqual(k8s_obj.status.conditions[0].type, 'StatefulSet')
        # V1StatefulSetStatus
        self.assertEqual(k8s_obj.status.replicas, 1)

    def test_storage_class(self):
        k8s_objs = self.transfromer.get_k8s_objs_from_yaml(
            ['storage-class.yaml'], self.yaml_path, ''
        )
        k8s_obj = k8s_objs[0].get('object')
        self.assertIsNotNone(k8s_obj)
        self.assertEqual(k8s_objs[0].get('namespace'), '')
        self.assertEqual(k8s_obj.kind, 'StorageClass')
        self.assertEqual(k8s_obj.api_version, 'storage.k8s.io/v1')
        # V1StorageClass
        self.assertEqual(k8s_obj.provisioner, 'kubernetes.io/no-provisioner')
        # V1TopologySelectorLabelRequirement
        self.assertEqual(k8s_obj.allowed_topologies[0].
                         match_label_expressions[0].key,
                         'failure-domain.beta.kubernetes.io/zone')
        self.assertEqual(k8s_obj.allowed_topologies[0].
                         match_label_expressions[0].values[0],
                         'us-central1-a')

    def test_subject_access_review(self):
        k8s_objs = self.transfromer.get_k8s_objs_from_yaml(
            ['subject-access-review.yaml'], self.yaml_path, ''
        )
        k8s_obj = k8s_objs[0].get('object')
        self.assertIsNotNone(k8s_obj)
        self.assertEqual(k8s_objs[0].get('namespace'), '')
        self.assertEqual(k8s_obj.kind, 'SubjectAccessReview')
        self.assertEqual(k8s_obj.api_version, 'authorization.k8s.io/v1')
        # V1SubjectAccessReviewStatus
        self.assertEqual(k8s_obj.status.allowed, True)
        # V1SubjectAccessReview
        self.assertIsNotNone(k8s_obj.spec)

    def test_token_review(self):
        k8s_objs = self.transfromer.get_k8s_objs_from_yaml(
            ['token-review.yaml'], self.yaml_path, ''
        )
        k8s_obj = k8s_objs[0].get('object')
        self.assertIsNotNone(k8s_obj)
        self.assertEqual(k8s_objs[0].get('namespace'), '')
        self.assertEqual(k8s_obj.kind, 'TokenReview')
        self.assertEqual(k8s_obj.api_version, 'authentication.k8s.io/v1')
        # V1TokenReview
        self.assertIsNotNone(k8s_obj.spec)

    def test_limit_range(self):
        k8s_objs = self.transfromer.get_k8s_objs_from_yaml(
            ['limit-range.yaml'], self.yaml_path, 'curryns'
        )
        k8s_obj = k8s_objs[0].get('object')
        self.assertIsNotNone(k8s_obj)
        self.assertEqual(k8s_objs[0].get('namespace'), 'curryns')
        self.assertEqual(k8s_obj.kind, 'LimitRange')
        self.assertEqual(k8s_obj.api_version, 'v1')
        # V1LimitRangeSpec
        self.assertIsNotNone(k8s_obj.spec.limits)
        self.assertIsNotNone(k8s_obj.spec.limits[0].type)

    def test_pod_template(self):
        k8s_objs = self.transfromer.get_k8s_objs_from_yaml(
            ['pod-template.yaml'], self.yaml_path, 'curryns'
        )
        k8s_obj = k8s_objs[0].get('object')
        self.assertIsNotNone(k8s_obj)
        self.assertEqual(k8s_objs[0].get('namespace'), 'curryns')
        self.assertEqual(k8s_obj.kind, 'PodTemplate')
        self.assertEqual(k8s_obj.api_version, 'v1')
        # V1AzureFileVolumeSource
        self.assertEqual(k8s_obj.template.spec.volumes[0].
                         azure_file.secret_name, 'azure-secret')
        # V1CephFSVolumeSource
        self.assertEqual(
            k8s_obj.template.spec.volumes[0].cephfs.monitors[0],
            '10.16.154.78:6789')
        # V1CinderVolumeSource
        self.assertEqual(
            k8s_obj.template.spec.volumes[0].cinder.volume_id,
            '90d6900d-808f-4ddb-a30e-5ef821f58b4e')
        # V1KeyToPath
        self.assertEqual(
            k8s_obj.template.spec.volumes[0].config_map.items[0].key,
            'log_level')
        self.assertEqual(
            k8s_obj.template.spec.volumes[0].config_map.items[0].path,
            'log_level')
        # V1CSIVolumeSource
        self.assertEqual(k8s_obj.template.spec.volumes[0].csi.driver,
                         'csi-nfsplugin')
        # V1DownwardAPIVolumeFile
        self.assertEqual(k8s_obj.template.spec.volumes[0].
                         downward_api.items[0].path, 'labels')
        # V1ObjectFieldSelector
        self.assertEqual(
            k8s_obj.template.spec.volumes[0].downward_api.items[0].
            field_ref.field_path, 'metadata.labels')
        # V1ResourceFieldSelector
        self.assertEqual(
            k8s_obj.template.spec.volumes[0].downward_api.items[0].
            resource_field_ref.resource, 'limits.cpu')
        # V1FlexVolumeSource
        self.assertEqual(
            k8s_obj.template.spec.volumes[0].flex_volume.driver,
            'kubernetes.io/lvm')
        # V1GCEPersistentDiskVolumeSource
        self.assertEqual(
            k8s_obj.template.spec.volumes[0].
            gce_persistent_disk.pd_name, 'my-data-disk')
        # V1GitRepoVolumeSource
        self.assertEqual(
            k8s_obj.template.spec.volumes[0].
            git_repo.repository, 'git@somewhere:me/my-git-repository.git')
        # V1GlusterfsVolumeSource
        self.assertEqual(
            k8s_obj.template.spec.volumes[0].glusterfs.endpoints,
            'glusterfs-cluster')
        self.assertEqual(
            k8s_obj.template.spec.volumes[0].glusterfs.path,
            'kube_vol')
        # V1HostPathVolumeSource
        self.assertEqual(
            k8s_obj.template.spec.volumes[0].host_path.path,
            '/var/local/aaa')
        # V1ISCSIVolumeSource
        self.assertEqual(
            k8s_obj.template.spec.volumes[0].iscsi.target_portal,
            '10.0.2.15:3260')
        self.assertEqual(
            k8s_obj.template.spec.volumes[0].iscsi.iqn,
            'iqn.2001-04.com.example:storage.kube.sys1.xyz')
        self.assertEqual(k8s_obj.template.spec.volumes[0].iscsi.lun, 0)
        # V1Volume
        self.assertEqual(k8s_obj.template.spec.volumes[0].name,
                         'curry-claim-volume')
        # V1NFSVolumeSource
        self.assertEqual(k8s_obj.template.spec.volumes[0].nfs.path, '/')
        self.assertEqual(
            k8s_obj.template.spec.volumes[0].nfs.server,
            'nfs-server.default.svc.cluster.local')
        # V1PersistentVolumeClaimVolumeSource
        self.assertEqual(
            k8s_obj.template.spec.volumes[0].
            persistent_volume_claim.claim_name, 'curry-pv-claim')
        # V1PhotonPersistentDiskVolumeSource
        self.assertEqual(
            k8s_obj.template.spec.volumes[0].
            photon_persistent_disk.pd_id, 'test')
        # V1PortworxVolumeSource
        self.assertEqual(
            k8s_obj.template.spec.volumes[0].
            portworx_volume.volume_id, 'pxvol')
        # V1ProjectedVolumeSource
        self.assertIsNotNone(k8s_obj.template.spec.volumes[0].
                             projected.sources)
        # V1QuobyteVolumeSource
        self.assertIsNotNone(
            k8s_obj.template.spec.volumes[0].
            quobyte.registry, 'test')
        self.assertIsNotNone(
            k8s_obj.template.spec.volumes[0].
            quobyte.volume, 'test')
        # V1RBDVolumeSource
        self.assertEqual(
            k8s_obj.template.spec.volumes[0].rbd.monitors[0],
            '10.16.154.78:6789')
        self.assertEqual(k8s_obj.template.spec.volumes[0].rbd.image, 'foo')
        # V1ScaleIOVolumeSource
        self.assertEqual(
            k8s_obj.template.spec.volumes[0].scale_io.gateway,
            'https://localhost:443/api')
        self.assertIsNotNone(
            k8s_obj.template.spec.volumes[0].scale_io.secret_ref)
        self.assertEqual(
            k8s_obj.template.spec.volumes[0].scale_io.system, 'scaleio')
        # V1VsphereVirtualDiskVolumeSource
        self.assertEqual(
            k8s_obj.template.spec.volumes[0].vsphere_volume.
            volume_path, '[DatastoreName] volumes/myDisk')
        # V1ServiceAccountTokenProjection
        self.assertEqual(
            k8s_obj.template.spec.volumes[0].
            projected.sources[0].service_account_token.path, 'test')

    def test_volume_attachment(self):
        k8s_objs = self.transfromer.get_k8s_objs_from_yaml(
            ['volume-attachment.yaml'], self.yaml_path, 'curryns'
        )
        k8s_obj = k8s_objs[0].get('object')
        self.assertIsNotNone(k8s_obj)
        self.assertEqual(k8s_objs[0].get('namespace'), 'curryns')
        self.assertEqual(k8s_obj.kind, 'VolumeAttachment')
        self.assertEqual(k8s_obj.api_version, 'storage.k8s.io/v1')
        # V1VolumeAttachment
        self.assertIsNotNone(k8s_obj.spec)
        # V1VolumeAttachmentSpec
        self.assertEqual(k8s_obj.spec.attacher, 'nginx')
        self.assertEqual(k8s_obj.spec.node_name, 'nginx')
        self.assertIsNotNone(k8s_obj.spec.source)
        # V1VolumeAttachmentStatus
        self.assertEqual(k8s_obj.status.attached, True)

    def test_bindings(self):
        k8s_objs = self.transfromer.get_k8s_objs_from_yaml(
            ['bindings.yaml'], self.yaml_path, 'curryns'
        )
        k8s_obj = k8s_objs[0].get('object')
        self.assertIsNotNone(k8s_obj)
        self.assertEqual(k8s_objs[0].get('namespace'), 'curryns')
        self.assertEqual(k8s_obj.kind, 'Binding')
        self.assertEqual(k8s_obj.api_version, 'v1')
        # V1Binding
        self.assertIsNotNone(k8s_obj.target)

    def test_controller_revision(self):
        k8s_objs = self.transfromer.get_k8s_objs_from_yaml(
            ['controller-revision.yaml'], self.yaml_path, 'curryns'
        )
        k8s_obj = k8s_objs[0].get('object')
        self.assertIsNotNone(k8s_obj)
        self.assertEqual(k8s_objs[0].get('namespace'), 'curryns')
        self.assertEqual(k8s_obj.kind, 'ControllerRevision')
        self.assertEqual(k8s_obj.api_version, 'apps/v1')
        # V1ControllerRevision
        self.assertEqual(k8s_obj.revision, 1)

    def test_transform(self):
        container_obj = tosca_kube_object.Container(
            config='config:abc\nconfig2:bcd',
            num_cpus=2,
            mem_size=10,
            name='container'
        )
        tosca_kube_objects = [tosca_kube_object.ToscaKubeObject(
            namespace='namespace',
            name='name',
            containers=[container_obj],
            mapping_ports=["123"],
            labels={}
        )]
        kubernetes_objects = self.transfromer.transform(tosca_kube_objects)
        self.assertEqual(kubernetes_objects['namespace'], 'namespace')
        self.assertEqual(
            kubernetes_objects['objects'][0].data, {
                'config': 'abc', 'config2': 'bcd'})

    @mock.patch.object(client.CoreV1Api, 'create_namespaced_config_map')
    @mock.patch.object(client.AppsV1Api, 'create_namespaced_deployment')
    @mock.patch.object(client.CoreV1Api, 'create_namespaced_service')
    def test_deploy(
            self,
            mock_create_namespaced_config_map,
            mock_create_namespaced_deployment,
            mock_create_namespaced_service):
        mock_create_namespaced_config_map.return_value = ""
        mock_create_namespaced_deployment.return_value = ""
        mock_create_namespaced_service.return_value = ""
        container_obj = tosca_kube_object.Container(
            config='config:abc\nconfig2:bcd',
            num_cpus=2,
            mem_size=10,
            name='container'
        )
        tosca_kube_objects = [tosca_kube_object.ToscaKubeObject(
            namespace='namespace',
            name='name',
            containers=[container_obj],
            mapping_ports=["123"],
            labels={}
        )]
        kubernetes_objects = self.transfromer.transform(tosca_kube_objects)
        result = self.transfromer.deploy(kubernetes_objects)
        self.assertEqual(result, 'namespace,name')



# Copyright (C) 2021 Nippon Telegraph and Telephone Corporation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from dateutil import parser

from tacker.sol_refactored.common import exceptions as sol_ex
from tacker.sol_refactored.controller import vnflcm_view
from tacker.tests import base


class FakeField(object):

    def __init__(self, nullable):
        self.nullable = nullable


class TestFilterExpr(base.BaseTestCase):

    def test_match_datetime(self):
        fexp = vnflcm_view.FilterExpr('lte',
            ['fooTime'], ['2021-08-24T02:26:46.796695+00:00'])
        val = {'fooTime': parser.isoparse("2021-08-11T08:09:10+00:00")}
        self.assertTrue(fexp.match(val))

        val2 = {'fooTime': parser.isoparse("2021-09-01T08:09:10+00:00")}
        self.assertFalse(fexp.match(val2))

    def test_match_int(self):
        fexp = vnflcm_view.FilterExpr('in',
            ['fooNum'], ['11', '13', '17', '19', '23'])
        self.assertTrue(fexp.match(dict(fooNum=11)))
        self.assertFalse(fexp.match(dict(fooNum=15)))

    def test_match_bool(self):
        fexp = vnflcm_view.FilterExpr('eq',
            ['foo', 'bar'], ['false'])
        self.assertFalse(fexp.match({'foo': {'bar': True}}))

        fexp2 = vnflcm_view.FilterExpr('eq',
            ['foo', 'bar'], ['true'])
        self.assertTrue(fexp2.match({'foo': {'bar': True}}))

        fexp3 = vnflcm_view.FilterExpr('eq',
            ['foo', 'bar'], ['invalid'])
        self.assertRaises(sol_ex.InvalidAttributeFilter,
                          fexp3.match,
                          {'foo': {'bar': True}})

    def test_match_key(self):
        fexp = vnflcm_view.FilterExpr('eq',
            ['foo', vnflcm_view.KeyAttribute()], ['abc'])
        self.assertFalse(fexp.match({'foo': {'bar': True}}))
        self.assertTrue(fexp.match({'foo': {'abc': False}}))

    def test_match_invalid(self):
        fexp = vnflcm_view.FilterExpr('eq',
            ['foo', 'bar', vnflcm_view.KeyAttribute()], ['abc'])
        self.assertRaises(sol_ex.InvalidAttributeFilter,
                          fexp.match, {'foo': 1})
        self.assertRaises(sol_ex.InvalidAttributeFilter,
                          fexp.match, {'foo': [1, 2, 3]})


class TestAttributeSelector(base.BaseTestCase):

    def test_invalid_params(self):
        self.assertRaises(sol_ex.InvalidAttributeSelector,
                          vnflcm_view.AttributeSelector,
                          [], all_fields='1', exclude_default='1')

        self.assertRaises(sol_ex.InvalidAttributeSelector,
                          vnflcm_view.AttributeSelector,
                          [], fields='a', exclude_fields='b')

        self.assertRaises(sol_ex.InvalidAttributeSelector,
                          vnflcm_view.AttributeSelector,
                          [], exclude_default='1', exclude_fields='b')

    def test_filter_default(self):
        selector = vnflcm_view.AttributeSelector(
            ['foo', 'hoge/foo1', 'bar'])
        obj = mock.NonCallableMagicMock()
        obj.fields.__getitem__.return_value = FakeField(True)
        r = selector.filter(obj, {'foo': 1, 'bar': 2, 'baz': 3})
        self.assertEqual(r, {'baz': 3})
        obj.fields.__getitem__.assert_called_with('bar')

    def test_filter_exclude_default(self):
        selector = vnflcm_view.AttributeSelector(['foo', 'bar'],
                                                 exclude_default='1')
        obj = mock.NonCallableMagicMock()
        obj.fields.__getitem__.return_value = FakeField(True)
        r = selector.filter(obj, {'foo': 1, 'bar': 2, 'baz': 3})
        self.assertEqual(r, {'baz': 3})

    def test_filter_default_not_omittable(self):
        selector = vnflcm_view.AttributeSelector(['foo', 'bar'])
        obj = mock.NonCallableMagicMock()
        obj.fields.__getitem__.return_value = FakeField(False)
        r = selector.filter(obj, {'foo': 1, 'bar': 2})
        self.assertEqual(r, {'foo': 1, 'bar': 2})

    def test_filter_all_fields(self):
        selector = vnflcm_view.AttributeSelector(['foo', 'bar'])
        obj = mock.NonCallableMagicMock()
        obj.fields.__getitem__.return_value = FakeField(True)
        odict = {'foo': 1, 'bar': 2, 'baz': 3}
        r = selector.filter(obj, odict)
        self.assertEqual(r, odict)

    def test_filter_exclude_fields(self):
        selector = vnflcm_view.AttributeSelector(['foo', 'bar'],
                                                 exclude_fields='bar,baz')
        obj = mock.NonCallableMagicMock()
        obj.fields.__getitem__.return_value = FakeField(True)
        r = selector.filter(obj, {'foo': 1, 'bar': 2, 'baz': 3})
        self.assertEqual(r, {'foo': 1})

    def test_filter_fields(self):
        selector = vnflcm_view.AttributeSelector(['foo', 'bar'],
                                                 exclude_default='1',
                                                 fields='bar')
        obj = mock.NonCallableMagicMock()
        obj.fields.__getitem__.return_value = FakeField(True)
        r = selector.filter(obj, {'foo': 1, 'bar': 2, 'baz': 3})
        self.assertEqual(r, {'bar': 2, 'baz': 3})


class TestBaseViewBuilder(base.BaseTestCase):

    def test_parse_filter(self):
        builder = vnflcm_view.BaseViewBuilder()
        f1 = builder.parse_filter("(eq,foo/bar,abc)")
        self.assertEqual(len(f1), 1)
        self.assertEqual(f1[0].attr, ['foo', 'bar'])

        f2 = builder.parse_filter("(eq,foo/bar,')1');(neq,baz,'''a')")
        self.assertEqual(len(f2), 2)
        self.assertEqual(f2[0].attr, ['foo', 'bar'])
        self.assertEqual(f2[0].values, [')1'])
        self.assertEqual(f2[1].attr, ['baz'])
        self.assertEqual(f2[1].values, ["'a"])

        f3 = builder.parse_filter("(eq,~01/c~1~a/~be,10)")
        self.assertEqual(len(f3), 1)
        self.assertEqual(f3[0].attr, ['~1', 'c/,', '@e'])

        f4 = builder.parse_filter("(in,foo,'a','bc');(cont,bar,'def','ghi')")
        self.assertEqual(len(f4), 2)
        self.assertEqual(len(f4[0].values), 2)
        self.assertEqual(len(f4[1].values), 2)

        f5 = builder.parse_filter("(eq,foo/@key,'abc')")
        self.assertEqual(len(f5), 1)
        self.assertEqual(len(f5[0].attr), 2)
        self.assertEqual(f5[0].attr[0], 'foo')
        self.assertIsInstance(f5[0].attr[1], vnflcm_view.KeyAttribute)
        self.assertEqual(len(f5[0].values), 1)

    def test_parse_filter_invalid(self):
        builder = vnflcm_view.BaseViewBuilder()
        self.assertRaises(sol_ex.InvalidAttributeFilter,
                          builder.parse_filter,
                          "(le,foo/bar,abc)")

        self.assertRaises(sol_ex.InvalidAttributeFilter,
                          builder.parse_filter,
                          "(gt,foo/bar)")

        self.assertRaises(sol_ex.InvalidAttributeFilter,
                          builder.parse_filter,
                          "(gt,foo,1,2)")



# Copyright 2018 OpenStack Foundation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

# openstack infra constants

STACK_CREATE_IN_PROGRESS = "CREATE_IN_PROGRESS"
STACK_CREATE_COMPLETE = "CREATE_COMPLETE"
STACK_UPDATE_IN_PROGRESS = "UPDATE_IN_PROGRESS"
STACK_UPDATE_COMPLETE = "UPDATE_COMPLETE"
STACK_DELETE_IN_PROGRESS = "DELETE_IN_PROGRESS"
STACK_DELETE_COMPLETE = "DELETE_COMPLETE"



# -*- coding: utf-8 -*-
#
# Copyright (c) 2020 OpenStack Foundation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import abc

from oslo_utils import strutils
from oslo_utils import timeutils
from oslo_utils import uuidutils

from tacker.common import exceptions as exception


registered_filters = {}
SUPPORTED_OP_ONE = ['eq', 'neq', 'gt', 'lt', 'gte', 'lte']
SUPPORTED_OP_MULTI = ['in', 'nin', 'cont', 'ncont']


class BaseFilter(object, metaclass=abc.ABCMeta):
    """Abstract base class for Filter classes."""

    @abc.abstractmethod
    def __str__(self):
        """String representation of the filter tree rooted at this node."""

        pass

    @abc.abstractmethod
    def __call__(self, target):
        """Triggers if instance of the class is called.

        Performs the checks against operators, attribute and datatype
        of the value. Raises exception if it's invalid and finally attribute
        is mapped to the database model that's present in the target dict.
        """

        pass


class Filter(BaseFilter):
    SUPPORTED_OPERATORS = None

    FILTER_OPERATOR_SPEC_MAPPING = {
        'eq': '==',
        'neq': '!=',
        'in': 'in',
        'nin': 'not_in',
        'gt': '>',
        'gte': '>=',
        'lt': '<',
        'lte': '<=',
        'cont': 'in',
        'ncont': 'not_in'
    }

    OPERATOR_SUPPORTED_DATA_TYPES = {
        'eq': ['uuid', 'string', 'number', 'enum', 'boolean',
               'key_value_pair'],
        'neq': ['uuid', 'string', 'number', 'enum', 'boolean',
                'key_value_pair'],
        'in': ['uuid', 'string', 'number', 'enum', 'key_value_pair'],
        'nin': ['uuid', 'string', 'number', 'enum', 'key_value_pair'],
        'gt': ['uuid', 'string', 'number', 'datetime', 'key_value_pair'],
        'gte': ['uuid', 'string', 'number', 'datetime', 'key_value_pair'],
        'lt': ['uuid', 'string', 'number', 'datetime', 'key_value_pair'],
        'lte': ['uuid', 'string', 'number', 'datetime', 'key_value_pair'],
        'cont': ['uuid', 'string', 'key_value_pair'],
        'ncont': ['uuid', 'string', 'key_value_pair'],
    }

    def __init__(self, operator, attribute, values):
        self.operator = operator
        self.attribute = attribute
        self.values = values

    def __str__(self):
        """Return a string representation of this filter."""

        return '%s,%s,%s' % (self.operator, self.attribute,
            ",".join(self.values))

    def _attribute_special_field(self, target):
        """Check if an attribute is a special field in the target

        Look for attributes in the target that ends with '*' as
        these are special attributes whose type could be 'key_value'
        which requires special treatment. For example
        if attribute in target is 'userDefinedData/*' and if self.attribute
        is userDefinedData/key1, then it's valid even though there is no
        exact match in the target because key/value pair values are
        dynamic.
        """
        special_attributes = [attribute for attribute in target.keys() if '*'
            in attribute]

        for attribute in special_attributes:
            field = attribute.split('*')[0]
            if self.attribute.startswith(field):
                return attribute

    def _validate_operators(self):
        if not self.operator:
            msg = ("Rule '%(rule)s' cannot contain operator")
            raise exception.ValidationError(msg % {"rule": self})

        if self.SUPPORTED_OPERATORS and self.operator not in \
                self.SUPPORTED_OPERATORS:
            msg = ("Rule '%(rule)s' contains invalid operator "
                   "'%(operator)s'")
            raise exception.ValidationError(msg % {"rule": self,
                "operator": self.operator})

    def _validate_attribute_name(self, target):
        if not self.attribute:
            msg = ("Rule '%(rule)s' doesn't contain attribute name")
            raise exception.ValidationError(msg % {"rule": self})

        if '*' in self.attribute:
            msg = ("Rule '%(rule)s' contains invalid attribute name "
                   "'%(attribute)s'")
            raise exception.ValidationError(msg % {"rule": self,
                "attribute": self.attribute})

        if target and self.attribute not in target:
            if not self._attribute_special_field(target):
                msg = ("Rule '%(rule)s' contains invalid attribute name "
                       "'%(attribute)s'")
                raise exception.ValidationError(msg % {"rule": self,
                    "attribute": self.attribute})

    def _handle_string(self, value):
        if value[0] == "'" and value[-1] == "'":
            value = value.strip("'")
            # The logic below enforces single quotes to be in pairs.
            # Raises exception otherwise. It also replaces a pair of
            # single quotes with one single quote.
            # NFV_SOL013 Section 5.2.2
            num_quotes = value.count("'")
            value = value.replace("''", "'")
            if (value.count("'") * 2) != num_quotes:
                msg = ("Rule '%(rule)s' value doesn't have single "
                       "quotes in pairs")
                raise exception.ValidationError(msg % {"rule": self})
        elif any(c in value for c in [",", ")", "'"]):
            msg = ("Rule '%(rule)s' value must be enclosed in "
                   "single quotes when it contains either of "
                   "comma, single quote, closing bracket")
            raise exception.ValidationError(msg % {"rule": self})
        return value

    def _handle_values(self, target):
        special_attribute = self._attribute_special_field(target)
        if special_attribute:
            attribute_info = target.get(special_attribute)
        else:
            attribute_info = target.get(self.attribute)

        if attribute_info[1] in ['string', 'key_value_pair']:
            values = [self._handle_string(v) for v in self.values]
            self.values = values

    def _validate_data_type(self, target):
        if not self.values:
            msg = ("Rule '%(rule)s' contains empty value")
            raise exception.ValidationError(msg % {"rule": self})

        special_attribute = self._attribute_special_field(target)
        if special_attribute:
            attribute_info = target.get(special_attribute)
        else:
            attribute_info = target.get(self.attribute)

        for value in self.values:
            error = False
            if attribute_info[1] == 'string' and not isinstance(value, str):
                error = True
            elif attribute_info[1] == 'number':
                if not strutils.is_int_like(value):
                    error = True
            elif attribute_info[1] == 'uuid':
                if not uuidutils.is_uuid_like(value):
                    error = True
            elif attribute_info[1] == 'datetime':
                try:
                    timeutils.parse_isotime(value)
                except ValueError:
                    error = True
            elif attribute_info[1] == 'enum':
                if value not in attribute_info[3]:
                    msg = ("Rule '%(rule)s' contains data type '%(type)s' "
                           "with invalid value. It should be one of "
                           "%(valid_value)s")
                    raise exception.ValidationError(msg % {"rule": self,
                        "valid_value": ",".join(attribute_info[3]),
                        'type': attribute_info[1]})

            if error:
                msg = ("Rule '%(rule)s' contains invalid data type for value "
                       "'%(value)s'. The data type should be '%(type)s'")
                raise exception.ValidationError(msg % {"rule": self,
                    "value": value,
                    'type': attribute_info[1]})

            # Also, check whether the data type is supported by operator
            if attribute_info[1] not in \
                    self.OPERATOR_SUPPORTED_DATA_TYPES.get(self.operator):
                msg = ("Rule '%(rule)s' contains operator '%(operator)s' "
                       "which doesn't support data type '%(type)s' for "
                       "attribute '%(attribute)s'")
                raise exception.ValidationError(msg % {"rule": self,
                                "operator": self.operator,
                                'type': attribute_info[1],
                                'attribute': self.attribute})

    def generate_expression(self, target, multiple_values=False):
        special_attribute = self._attribute_special_field(target)
        if special_attribute:
            attribute_info = target.get(special_attribute)
        else:
            attribute_info = target.get(self.attribute)

        attributes = attribute_info[0].split('.')
        key_token = self.attribute.split('/')[-1]
        if attribute_info[1] == 'key_value_pair':
            filter_spec = []
            expression_key = {'field': attribute_info[2]['key_column'],
                'model': attribute_info[2]['model'],
                'value': key_token,
                'op': self.FILTER_OPERATOR_SPEC_MAPPING.get(self.operator)}
            expression_value = {'field': attribute_info[2]['value_column'],
                'model': attribute_info[2]['model'],
                'value': self.values if multiple_values else self.values[0],
                'op': self.FILTER_OPERATOR_SPEC_MAPPING.get(self.operator)}
            filter_spec.append(expression_key)
            filter_spec.append(expression_value)
            expression = {'and': filter_spec}
        else:
            expression = {'field': attributes[-1],
                'model': attribute_info[2],
                'value': self.values if multiple_values else self.values[0],
                'op': self.FILTER_OPERATOR_SPEC_MAPPING.get(self.operator)}

        return expression


class AndFilter(BaseFilter):
    def __init__(self, filter_rule):
        self.filter_rules = filter_rule

    def __str__(self):
        """Return a string representation of this filter."""

        return '(%s)' % ' and '.join(str(r) for r in self.filter_rules)

    def __call__(self, target):
        """Run through this filter and maps it to the database model

        :returns
            A dict containing list of filter-specs required by
            sqlalchemy-filter.
        Example::

        filter=(eq,onboardingState,'onboarded');(eq,softwareImages/size, 10)

        Result would be:
        {
          'and': [
              {
                  'field': 'onboarding_state', 'model': 'Foo',
                  'value': "'onboarded'", 'op': '=='
              },
              {
                 'field': 'size', 'model': 'Foo',
                 'value': '10', 'op': '=='
              }
          ]
        }
        """
        filter_spec = []
        for filter_rule in self.filter_rules:
            result = filter_rule(target)
            filter_spec.append(result)

        return {'and': filter_spec}

    def add_filter_rule(self, filter_rule):
        """Adds filter rule to be tested.

        Allows addition of another filter rule to the list of filter rules
        that will be tested.

        :returns: self
        :rtype: :class:`.AndFilter`
        """

        self.filter_rules.append(filter_rule)
        return self


def register(name, func=None):
    # Perform the actual decoration by registering the function or
    # class.  Returns the function or class for compliance with the
    # decorator interface.
    def decorator(func):
        registered_filters[name] = func
        return func

    # If the function or class is given, do the registration
    if func:
        return decorator(func)

    return decorator


@register('simple_filter_expr_one')
class SimpleFilterExprOne(Filter):

    SUPPORTED_OPERATORS = SUPPORTED_OP_ONE

    def __call__(self, target):
        """Run through this filter and maps it to the database model

        :returns
            A dict containing list of filter-specs required by
            sqlalchemy-filter.
        Example::
        operator=eq, attribute=onBoardingState, and value='onboarded', then
        it would be mapped to following expression.
            {
                'field': 'onboarding_state', -> Mapped to the version field
                'model': 'Foo', -> Mapped to database model
                'value': "onboarded", -> Value to be used for filtering
                                         records
                'op': '==', -> Operator for comparison
            },
        """

        self._validate_operators()
        self._validate_attribute_name(target)
        self._validate_data_type(target)
        self._handle_values(target)

        return self.generate_expression(target, multiple_values=False)

    def _validate_operators(self):
        super(SimpleFilterExprOne, self)._validate_operators()
        if self.values and isinstance(self.values, list) and \
                len(self.values) > 1:
            msg = _("Rule '%(rule)s' contains operator '%(operator)s' "
                    "which supports only one value, but multiple values "
                    "'%(values)s' are provided")
            raise exception.ValidationError(msg % {"rule": self,
                            "operator": self.operator,
                            'values': ",".join(self.values)})


@register('simple_filter_expr_multi')
class SimpleFilterExprMulti(Filter):

    SUPPORTED_OPERATORS = SUPPORTED_OP_MULTI

    def __call__(self, target):
        """Run through this filter and maps it to the database model

        This filter is exactly same as SimpleFilterExprOne, except
        it supports different operators like 'in'|'nin'|'cont'|'ncont'
        which contains more than one value in the list.

        :returns
            A dict containing list of filter-specs required by
            sqlalchemy-filter.
        Example::
        operator=in, attribute=softwareImages/size, and value=[10,20]',
        then it would be mapped to following expression.
            {
                'field': 'size', -> Mapped to the version field
                'model': 'Foo', -> Mapped to database model
                'value': [10,20], -> Value to be used for filtering
                                     records
                'op': 'in', -> Attribute equal to one of the values in the
                               list ("in set" relationship)
            },
        """
        self._validate_operators()
        self._validate_attribute_name(target)
        self._validate_data_type(target)
        self._handle_values(target)

        return self.generate_expression(target, multiple_values=True)



# All Rights Reserved.
#
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os
import re
import tempfile

from cryptography import fernet
from kubernetes import client
from kubernetes.client import api_client
from oslo_config import cfg
from oslo_log import log as logging

LOG = logging.getLogger(__name__)
CONF = cfg.CONF


class KubernetesHTTPAPI(object):

    def get_k8s_client(self, auth_plugin):
        config = client.Configuration()
        config.host = auth_plugin['auth_url']
        if ('username' in auth_plugin) and ('password' in auth_plugin)\
                and (auth_plugin['password'] is not None):
            config.username = auth_plugin['username']
            config.password = auth_plugin['password']
            basic_token = config.get_basic_auth_token()
            config.api_key['authorization'] = basic_token
        if 'bearer_token' in auth_plugin:
            config.api_key_prefix['authorization'] = 'Bearer'
            config.api_key['authorization'] = auth_plugin['bearer_token']
        ca_cert_file = auth_plugin.get('ca_cert_file')
        if ca_cert_file is not None:
            config.ssl_ca_cert = ca_cert_file
            config.verify_ssl = True
        else:
            config.verify_ssl = False
        k8s_client = api_client.ApiClient(configuration=config)
        return k8s_client

    def get_k8s_client_dict(self, auth):
        k8s_client_dict = {
            'v1': self.get_core_v1_api_client(auth),
            'apiregistration.k8s.io/v1':
            self.get_api_registration_v1_api_client(auth),
            'apps/v1': self.get_app_v1_api_client(auth),
            'authentication.k8s.io/v1':
            self.get_authentication_v1_api_client(auth),
            'authorization.k8s.io/v1':
            self.get_authorization_v1_api_client(auth),
            'autoscaling/v1': self.get_scaling_api_client(auth),
            'batch/v1': self.get_batch_v1_api_client(auth),
            'coordination.k8s.io/v1':
            self.get_coordination_v1_api_client(auth),
            'networking.k8s.io/v1':
            self.get_networking_v1_api_client(auth),
            'rbac.authorization.k8s.io/v1':
            self.get_rbac_authorization_v1_api_client(auth),
            'scheduling.k8s.io/v1':
            self.get_scheduling_v1_api_client(auth),
            'storage.k8s.io/v1':
            self.get_storage_v1_api_client(auth)
        }
        return k8s_client_dict

    def get_extension_api_client(self, auth):
        k8s_client = self.get_k8s_client(auth_plugin=auth)
        return client.ExtensionsV1beta1Api(api_client=k8s_client)

    def get_core_v1_api_client(self, auth):
        k8s_client = self.get_k8s_client(auth_plugin=auth)
        return client.CoreV1Api(api_client=k8s_client)

    def get_core_api_client(self, auth):
        k8s_client = self.get_k8s_client(auth_plugin=auth)
        return client.CoreApi(api_client=k8s_client)

    def get_scaling_api_client(self, auth):
        k8s_client = self.get_k8s_client(auth_plugin=auth)
        return client.AutoscalingV1Api(api_client=k8s_client)

    def get_app_v1_api_client(self, auth):
        k8s_client = self.get_k8s_client(auth_plugin=auth)
        return client.AppsV1Api(api_client=k8s_client)

    def get_api_registration_v1_api_client(self, auth):
        k8s_client = self.get_k8s_client(auth_plugin=auth)
        return client.ApiregistrationV1Api(api_client=k8s_client)

    def get_authentication_v1_api_client(self, auth):
        k8s_client = self.get_k8s_client(auth_plugin=auth)
        return client.AuthenticationV1Api(api_client=k8s_client)

    def get_authorization_v1_api_client(self, auth):
        k8s_client = self.get_k8s_client(auth_plugin=auth)
        return client.AuthorizationV1Api(api_client=k8s_client)

    def get_batch_v1_api_client(self, auth):
        k8s_client = self.get_k8s_client(auth_plugin=auth)
        return client.BatchV1Api(api_client=k8s_client)

    def get_coordination_v1_api_client(self, auth):
        k8s_client = self.get_k8s_client(auth_plugin=auth)
        return client.CoordinationV1Api(api_client=k8s_client)

    def get_networking_v1_api_client(self, auth):
        k8s_client = self.get_k8s_client(auth_plugin=auth)
        return client.NetworkingV1Api(api_client=k8s_client)

    def get_rbac_authorization_v1_api_client(self, auth):
        k8s_client = self.get_k8s_client(auth_plugin=auth)
        return client.RbacAuthorizationV1Api(api_client=k8s_client)

    def get_scheduling_v1_api_client(self, auth):
        k8s_client = self.get_k8s_client(auth_plugin=auth)
        return client.SchedulingV1Api(api_client=k8s_client)

    def get_storage_v1_api_client(self, auth):
        k8s_client = self.get_k8s_client(auth_plugin=auth)
        return client.StorageV1Api(api_client=k8s_client)

    @staticmethod
    def create_ca_cert_tmp_file(ca_cert):
        file_descriptor, file_path = tempfile.mkstemp()
        ca_cert = re.sub(r'\s', '\n', ca_cert)
        ca_cert = re.sub(r'BEGIN\nCERT', r'BEGIN CERT', ca_cert)
        ca_cert = re.sub(r'END\nCERT', r'END CERT', ca_cert)
        try:
            with open(file_path, 'w') as f:
                f.write(ca_cert)
                LOG.debug('ca cert temp file successfully stored in %s',
                          file_path)
        except IOError:
            raise Exception('Failed to create %s file', file_path)
        return file_descriptor, file_path

    @staticmethod
    def close_tmp_file(file_descriptor, file_path):
        os.close(file_descriptor)
        os.remove(file_path)

    def create_fernet_key(self):
        fernet_key = fernet.Fernet.generate_key()
        fernet_obj = fernet.Fernet(fernet_key)
        return fernet_key, fernet_obj
